import https from "https"
import fs from "fs"
const commands = new Map()

export default class Handler {
	#client; #commandsFolder; #consoleLogging
	constructor({ client, commandsFolder, consoleLogging, checkUpdate=true } ) {
		this.#client = client
		this.#consoleLogging = consoleLogging
		this.#commandsFolder = commandsFolder
		this.#importCommands(true)

		if (checkUpdate) {
			console.log("Checking for update...")
			https.get("https://registry.npmjs.com/discord.js-commands-handler", (res) => {
				let body = ""
				res.on("data", data => body += data)
				res.on("end", () => {
					if ((JSON.parse(body))["dist-tags"].latest == JSON.parse(fs.readFileSync("node_modules/discord.js-commands-handler/package.json")).version) {
						console.log("Your module is in the latest version!")
						console.log("\nYou can cancel the update check by defining the Handler as `new Handler({ ... , checkUpdate: false })`\n")
					} else {
						console.log(`Your module is not the latest version. Latest version of the module is ${(JSON.parse(body))["dist-tags"].latest}\nYou can update your module by writing \`npm install discord.js-commands-handler@latest\``)
						console.log("\nYou can cancel the update check by defining the Handler as `new Handler({ ... , checkUpdate: false })`\n")
					}
				})
			})
		}
	}

	async setCommand(command={ type: 1, name, description, cooldown, guild, permissions, options, execute() {}, componentInteraction() {}, autocompleteInteraction() {}, modalInteraction() {} }) {
		await fs.writeFileSync(`${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}/${command.name}.js`, `export default {${command.type ? `\n\ttype: ${command.type},` : ""}${command.name ? `\n\tname: "${command.name}",` : ""}${command.description ? `\n\tdescription: "${command.description}",` : ""}${command.cooldown ? `\n\tcooldown: "${command.cooldown}",` : ""}${command.guild ? `\n\tguild: ${command.guild},` : ""}${command.permissions ? `\n\tpermissions: ${JSON.stringify(command.permissions)},` : ""}${command.options ? `\n\toptions: ${JSON.stringify(command.options)},` : ""}${command.execute.toString() ? `\n\t${command.execute.toString()},` : ""}${command.componentInteraction?.toString() ? `\n\t${command.componentInteraction?.toString()},` : ""}${command.autocompleteInteraction?.toString() ? `\n\t${command.autocompleteInteraction?.toString()},` : ""}${command.modalInteraction?.toString() ? `\n\t${command.modalInteraction?.toString()}` : ""}\n}`)
		await import(`file://${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}/${command.name}.js`).then(file => commands.set(file.default.name, file.default))
		this.#client.guilds.cache.forEach(guild => {
			if (command.guild == guild.id) {
				guild.commands.create({
					type: command.type,
					name: command.name,
					description: command.description,
					defaultMemberPermissions: command.permissions || "UseApplicationCommands",
					options: command.options
				})
			} else {
				guild.commands.create({
					type: command.type,
					name: command.name,
					description: command.description,
					defaultMemberPermissions: command.permissions || "UseApplicationCommands",
					options: command.options
				})
			}

			if (this.#consoleLogging) {
				console.log(`Command synchronized > ${command.name[0].toUpperCase() + command.name.slice(1)}`)
			}
		}); console.log()
	}

	async getCommand(commandName, callback=command => {}) {
		if (!commands.size) await this.#importCommands()
		
		return callback(commands.get(commandName))
	}

	async hasCommand(commandName, callback=command => {}) {
		if (!commands.size) await this.#importCommands()
		
		return callback(commands.has(commandName))
	}

	async deleteCommand(commandName, callback = command => {}) {
		if (!commands.size) await this.#importCommands()

		commands.delete(commandName)
		this.#client.guilds.cache.forEach(async guild => guild.commands.delete(guild.commands.get(commandName)))
		fs.readdirSync(`${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}`).filter(async commandFile => {
			const file = await import(`file://${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}/${commandFile}`)
			if (file.default.name == commandName) {
				fs.unlinkSync(`${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}/${commandFile}`)
				return callback(file.default)
			}
		})
	}

	async synchronizeCommands() {
		this.#client.once("ready", () => {
			commands.forEach(command => {
				this.#client.guilds.cache.forEach(guild => {
					if (command.guild == guild.id) {
						guild.commands.create({
							type: command.type,
							name: command.name,
							description: command.description,
							defaultMemberPermissions: command.permissions || "UseApplicationCommands",
							options: command.options
						})
					} else {
						guild.commands.create({
							type: command.type,
							name: command.name,
							description: command.description,
							defaultMemberPermissions: command.permissions || "UseApplicationCommands",
							options: command.options
						})
					}

					guild.commands.fetch().then(guildCommands => {
						guildCommands.forEach(command => {
							if (!Array.from(commands.keys()).includes(command.name)) {
								guild.commands.delete(command.id)
							}
						})
					})
				})
			}); console.log()
		})

		this.#client.on('interactionCreate', async (interaction) => {
			if (interaction.isCommand()) {
				const command = commands.get(interaction.command.name)

				try {
					if (command.cooldown) {
						if (interaction.user.cooldowns) {
							if (interaction.user.cooldowns.get(command.name) - Date.now() > 0) {
								interaction.reply({
									embeds: [
										{
											title: '❌ Error ❌',
											description: `You can use this command again in **\`${Math.round((interaction.user.cooldowns.get(command.name)-Date.now())/100)/10}\`** seconds`,
											color: 0xff0000
										}
									]
								})
							} else {
								command.execute(this.#client, interaction, interaction.options)
								interaction.user.cooldowns.set(command.name, Date.now() + command.cooldown*1000)
							}
						} else {
							command.execute(this.#client, interaction, interaction.options)
							interaction.user.cooldowns = new Map()
							interaction.user.cooldowns.set(command.name, Date.now() + command.cooldown*1000)
						}
					} else {
						command.execute(this.#client, interaction, interaction.options)
					}
				} catch (error) {
					interaction.reply({
						embeds: [
							{
								title: '❌ Error ❌',
								description: 'An error occurred while running this command',
								color: 0xff0000
							}
						]
					})
					console.error(error)
				}
			} else if (interaction.isMessageComponent()) {
				const command = commands.get(interaction.customId.split(".")[0])

				try {
					command.componentInteraction(this.#client, interaction, interaction.customId.split(".")[1])
				} catch (error) {
					console.error(error)
				}
			} else if (interaction.isAutocomplete()) {
				const command = commands.get(interaction.command.name)

				try {
					command.autocompleteInteraction(this.#client, interaction, interaction.options.getFocused())
				} catch (error) {
					console.error(error)
				}
			} else if (interaction.isModalSubmit()) {
				const command = commands.get(interaction.customId.split(".")[0])

				try {
					command.modalInteraction(this.#client, interaction, interaction.customId.split(".")[1])
				} catch (error) {
					console.error(error)
				}
			}
		})

		this.#client.on('guildCreate', (guild) => {
			commands.forEach(command => {
				if (command.guild == guild.id) {
					guild.commands.create({
						type: command.type,
						name: command.name,
						description: command.description,
						defaultMemberPermissions: command.permissions || "UseApplicationCommands",
						options: command.options,
					})
				} else {
					guild.commands.create({
						type: command.type,
						name: command.name,
						description: command.description,
						defaultMemberPermissions: command.permissions || "UseApplicationCommands",
						options: command.options,
					})
				}
			})
		})
	}

	#importCommands(logCommand) {
		for (let commandFile of fs.readdirSync(`${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}`)) {
			import(`file://${process.cwd().replaceAll("\\", "/")}/${this.#commandsFolder}/${commandFile}`).then(file => {
				commands.set(file.default.name, file.default)

				setTimeout(() => {
					if (this.#consoleLogging && logCommand) {
						console.log(`Command synchronized > ${file.default.name[0].toUpperCase() + file.default.name.slice(1)}`)
					}
				}, 1000)
			})
		}
	}
}